from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.test import TestCase

from netbox_routing.models.bgp import *
from netbox_routing.tests.base import *
from netbox_routing.tests.bgp.base import *

__all__ = ('BGPRouterTestCase',)


class BGPRouterTestCase(ASNMixin, TestCase):
    model = BGPRouter

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        name = 'BGP Router 1'
        instance = self.model(
            name=name,
            asn=self.asn,
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), f'{name} ({self.asn})')
        self.assertEqual(instance.asn, self.asn)

    def test_unique_together(self):

        name = 'BGP Router 1'
        instance = self.model(
            name=name,
            asn=self.asn,
        )
        instance.full_clean()
        instance.save()


class BGPScopeTestCase(BGPRouterMixin, VRFMixin, TestCase):
    model = BGPScope

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            router=self.router,
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), f'{self.router}: Global VRF')
        self.assertEqual(instance.router, self.router)

    def test_model_with_vrf(self):
        instance = self.model(
            router=self.router,
            vrf=self.vrf,
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), f'{self.router}: {self.vrf}')
        self.assertEqual(instance.router, self.router)
        self.assertEqual(instance.vrf, self.vrf)

    def test_unique_together(self):
        instance = self.model(
            router=self.router,
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            router=self.router,
            vrf=self.vrf,
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            router=self.router,
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPAddressFamilyTestCase(BGPScopeMixin, TestCase):
    model = BGPAddressFamily

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(scope=self.scope, address_family='ipv4-unicast')
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.scope, self.scope)
        self.assertEqual(instance.address_family, 'ipv4-unicast')

    def test_model_invalid_af(self):
        instance = self.model(
            scope=self.scope,
            address_family='invalid',
        )
        with self.assertRaises(ValidationError):
            instance.full_clean()

    def test_unique_together(self):
        instance = self.model(
            scope=self.scope,
            address_family='ipv4-unicast',
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            scope=self.scope,
            address_family='ipv6-unicast',
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            scope=self.scope,
            address_family='ipv4-unicast',
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPPeerTestCase(
    BGPScopeMixin,
    AddressesMixin,
    TestCase,
):
    model = BGPPeer

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            name='Test Peer', scope=self.scope, peer=self.peer_address
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.scope, self.scope)
        self.assertEqual(instance.peer, self.peer_address)

    def test_unique_together(self):
        instance = self.model(
            name='Test Peer', scope=self.scope, peer=self.peer_address
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            name='Test Peer', scope=self.scope, peer=self.peer_address
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPPeerAddressFamilyTestCase(
    BGPPeerMixin,
    BGPAddressFamilyMixin,
    TestCase,
):
    model = BGPPeerAddressFamily

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            assigned_object=self.peer,
            address_family=self.af,
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.assigned_object, self.peer)
        self.assertEqual(instance.address_family, self.af)

    def test_unique_together(self):
        instance = self.model(
            assigned_object=self.peer,
            address_family=self.af,
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            assigned_object=self.peer,
            address_family=self.af,
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPPeerTemplateTestCase(
    BGPAddressFamilyMixin,
    TestCase,
):
    model = BGPPeerTemplate

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            name='Peer Template',
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), instance.name)

    def test_unique_together(self):
        instance = self.model(
            name='Peer Template',
            remote_as=self.asn,
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            name='Peer Template',
            remote_as=self.asn,
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPPolicyTemplateTestCase(
    BGPAddressFamilyMixin,
    TestCase,
):
    model = BGPPolicyTemplate

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            name='Policy Template',
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), instance.name)

    def test_unique_together(self):
        instance = self.model(
            name='Policy Template',
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            name='Policy Template',
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BGPSessionTemplateTestCase(
    BGPAddressFamilyMixin,
    TestCase,
):
    model = BGPSessionTemplate

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            name='Session Template',
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), instance.name)

    def test_unique_together(self):
        instance = self.model(
            name='Session Template',
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            name='Session Template',
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()


class BFDProfileTestCase(
    TestCase,
):
    model = BFDProfile

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def test_model(self):
        instance = self.model(
            name='Test BFD Profile X',
            min_rx_int=60,
            min_tx_int=60,
            multiplier=3,
        )
        instance.full_clean()
        instance.save()
        self.assertIsInstance(instance, self.model)
        self.assertEqual(instance.__str__(), instance.name)

    def test_unique_together(self):
        instance = self.model(
            name='Test BFD Profile X',
            min_rx_int=60,
            min_tx_int=60,
            multiplier=3,
        )
        instance.full_clean()
        instance.save()

        instance = self.model(
            name='Test BFD Profile X',
            min_rx_int=60,
            min_tx_int=60,
            multiplier=3,
        )

        with self.assertRaises(ValidationError):
            instance.full_clean()
        with self.assertRaises(IntegrityError):
            instance.save()



# Generated by Django 5.2.5 on 2026-01-16 23:40

import django.core.validators
import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0018_reset_bgp_model'),
    ]

    operations = [
        migrations.RenameField(
            model_name='prefixlistentry',
            old_name='type',
            new_name='action',
        ),
        migrations.RenameField(
            model_name='routemapentry',
            old_name='type',
            new_name='action',
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='family',
            field=models.PositiveSmallIntegerField(default=4),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='ge',
            field=models.PositiveSmallIntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(128),
                ],
            ),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='le',
            field=models.PositiveSmallIntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(128),
                ],
            ),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='prefix_list',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='prefix_list_entries',
                to='netbox_routing.prefixlist',
            ),
        ),
        migrations.AlterField(
            model_name='routemapentry',
            name='route_map',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='route_map_entries',
                to='netbox_routing.routemap',
            ),
        ),
        migrations.CreateModel(
            name='ASPath',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'ordering': ['name'],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ASPathEntry',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('sequence', models.PositiveSmallIntegerField()),
                ('action', models.CharField(max_length=6)),
                (
                    'aspath',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='aspath_entries',
                        to='netbox_routing.aspath',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                ('pattern', models.CharField(max_length=200)),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'ordering': ['aspath', 'sequence'],
            },
            bases=(
                netbox.models.deletion.DeleteMixin,
                models.Model,
            ),
        ),
        migrations.AlterField(
            model_name='prefixlist',
            name='name',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='routemap',
            name='name',
            field=models.CharField(max_length=100),
        ),
        migrations.AddConstraint(
            model_name='aspath',
            constraint=models.UniqueConstraint(
                django.db.models.functions.text.Lower('name'),
                name='netbox_routing_aspath_unique_name',
                violation_error_message='Name must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='aspathentry',
            constraint=models.UniqueConstraint(
                fields=('aspath', 'sequence'),
                name='netbox_routing_aspathentry_unique_aspath_sequence',
                violation_error_message='Prefix List sequence must be unique.',
            ),
        ),
        migrations.RemoveConstraint(
            model_name='prefixlistentry',
            name='netbox_routing_prefixlistentry_unique_prefixlist_sequence',
        ),
        migrations.RemoveConstraint(
            model_name='routemapentry',
            name='netbox_routing_routemapentry_unique_routemap_sequence',
        ),
        migrations.AddConstraint(
            model_name='prefixlistentry',
            constraint=models.UniqueConstraint(
                fields=('prefix_list', 'sequence'),
                name='netbox_routing_prefixlistentry_unique_prefixlist_sequence',
                violation_error_message='Prefix List sequence must be unique.',
            ),
        ),
        migrations.AddConstraint(
            model_name='routemapentry',
            constraint=models.UniqueConstraint(
                fields=('route_map', 'sequence'),
                name='netbox_routing_routemapentry_unique_routemap_sequence',
                violation_error_message='Route Map sequence must be unique.',
            ),
        ),
    ]



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable
from netbox_routing.models.community import *

__all__ = (
    'CommunityTable',
    'CommunityListTable',
    'CommunityListEntryTable',
)

from tenancy.tables import TenancyColumnsMixin


class CommunityTable(TenancyColumnsMixin, NetBoxTable):
    community = tables.Column(verbose_name=_('Name'), linkify=True)
    role = tables.Column(verbose_name=_('Role'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = Community
        fields = (
            'pk',
            'id',
            'community',
            'role',
            'status',
            'tenant_group',
            'tenant',
        )
        default_columns = (
            'pk',
            'id',
            'community',
        )


class CommunityListTable(TenancyColumnsMixin, NetBoxTable):
    name = tables.Column(verbose_name=_('Name'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = CommunityList
        fields = (
            'pk',
            'id',
            'name',
            'tenant_group',
            'tenant',
        )
        default_columns = (
            'pk',
            'id',
            'name',
        )


class CommunityListEntryTable(NetBoxTable):
    community_list = tables.Column(verbose_name=_('Community List'), linkify=True)
    community = tables.Column(verbose_name=_('Community'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = CommunityListEntry
        fields = (
            'pk',
            'id',
            'community_list',
            'action',
            'community',
        )
        default_columns = (
            'pk',
            'id',
            'community_list',
            'action',
            'community',
        )



import netaddr
from django.test import TestCase

from ipam.models import VRF
from utilities.testing import create_test_device

from netbox_routing.filtersets import *
from netbox_routing.models import *

__all__ = ('StaticRouteTestCase',)


class StaticRouteTestCase(TestCase):
    queryset = StaticRoute.objects.all()
    filterset = StaticRouteFilterSet

    @classmethod
    def setUpTestData(cls):
        devices = [
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
        ]
        vrf = VRF.objects.create(name='Test VRF')

        nh = netaddr.IPAddress('10.10.10.1')

        routes = (
            StaticRoute(name="Route 1", vrf=vrf, prefix='0.0.0.0/0', next_hop=nh),
            StaticRoute(
                name="Route 2",
                vrf=vrf,
                prefix='1.1.1.0/24',
                next_hop=netaddr.IPAddress('10.10.10.2'),
            ),
            StaticRoute(name="Route 3", prefix='0.0.0.0/0', next_hop=nh, metric=100),
        )

        StaticRoute.objects.bulk_create(routes)

        routes[0].devices.set([devices[0]])
        routes[1].devices.set([devices[0]])
        routes[2].devices.set([devices[1]])

    def test_q(self):
        params = {'q': 'Route 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_name(self):
        params = {'name': ['Route 1', 'Route 2']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_device(self):
        params = {'device': ['Device 1']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_vrf(self):
        params = {'vrf': ['Test VRF']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_prefix(self):
        params = {'prefix': '0.0.0.0/0'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_next_hop(self):
        params = {'next_hop': '10.10.10.1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_metric(self):
        params = {'metric': [1]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api.serializers import (
    OSPFInstanceSerializer,
    OSPFAreaSerializer,
    OSPFInterfaceSerializer,
)
from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface

__all__ = (
    'OSPFInstanceViewSet',
    'OSPFAreaViewSet',
    'OSPFInterfaceViewSet',
)


class OSPFInstanceViewSet(NetBoxModelViewSet):
    queryset = OSPFInstance.objects.all()
    serializer_class = OSPFInstanceSerializer
    filterset_class = filtersets.OSPFInstanceFilterSet


class OSPFAreaViewSet(NetBoxModelViewSet):
    queryset = OSPFArea.objects.all()
    serializer_class = OSPFAreaSerializer
    filterset_class = filtersets.OSPFAreaFilterSet


class OSPFInterfaceViewSet(NetBoxModelViewSet):
    queryset = OSPFInterface.objects.all()
    serializer_class = OSPFInterfaceSerializer
    filterset_class = filtersets.OSPFInterfaceFilterSet



from .static import *
from .objects import *
from .ospf import *
from .eigrp import *
from .bgp import *

__all__ = (
    # Staticroute
    'StaticRouteBulkEditForm',
    # OSPF
    'OSPFInstanceBulkEditForm',
    'OSPFInterfaceBulkEditForm',
    'OSPFAreaBulkEditForm',
    # EIGRP
    'EIGRPRouterBulkEditForm',
    'EIGRPAddressFamilyBulkEditForm',
    'EIGRPNetworkBulkEditForm',
    'EIGRPInterfaceBulkEditForm',
    # Route Objects
    'PrefixListEntryBulkEditForm',
    'RouteMapEntryBulkEditForm',
    'ASPathBulkEditForm',
    'ASPathEntryBulkEditForm',
    # BFD
    'BFDProfileBulkEditForm',
    # BGP
    'BGPPeerTemplateBulkEditForm',
    'BGPPolicyTemplateBulkEditForm',
    'BGPSessionTemplateBulkEditForm',
    'BGPRouterBulkEditForm',
    'BGPScopeBulkEditForm',
    'BGPAddressFamilyBulkEditForm',
    'BGPPeerBulkEditForm',
    'BGPPeerAddressFamilyBulkEditForm',
    'BGPSettingBulkEditForm',
)



# Generated by Django 5.0.8 on 2024-09-16 13:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0008_convert_to_primarymodel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staticroute',
            name='metric',
            field=models.PositiveSmallIntegerField(blank=True, default=1),
        ),
        migrations.AlterField(
            model_name='staticroute',
            name='permanent',
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
    ]



from django.urls import path, include

from utilities.urls import get_model_urls

from . import views  # noqa F401

app_name = 'netbox_routing'

urlpatterns = [
    path(
        'routes/static/', include(get_model_urls(app_name, 'staticroute', detail=False))
    ),
    path('routes/static/<int:pk>/', include(get_model_urls(app_name, 'staticroute'))),
    # OSPF
    path(
        'ospf/instance/',
        include(get_model_urls(app_name, 'ospfinstance', detail=False)),
    ),
    path('ospf/instance/<int:pk>/', include(get_model_urls(app_name, 'ospfinstance'))),
    path('ospf/area/', include(get_model_urls(app_name, 'ospfarea', detail=False))),
    path('ospf/area/<int:pk>/', include(get_model_urls(app_name, 'ospfarea'))),
    path(
        'ospf/interface/',
        include(get_model_urls(app_name, 'ospfinterface', detail=False)),
    ),
    path(
        'ospf/interface/<int:pk>/', include(get_model_urls(app_name, 'ospfinterface'))
    ),
    # EIGRP
    path(
        'eigrp/router/', include(get_model_urls(app_name, 'eigrprouter', detail=False))
    ),
    path('eigrp/router/<int:pk>/', include(get_model_urls(app_name, 'eigrprouter'))),
    path(
        'eigrp/address-family/',
        include(get_model_urls(app_name, 'eigrpaddressfamily', detail=False)),
    ),
    path(
        'eigrp/address-family/<int:pk>/',
        include(get_model_urls(app_name, 'eigrpaddressfamily')),
    ),
    path(
        'eigrp/network/',
        include(get_model_urls(app_name, 'eigrpnetwork', detail=False)),
    ),
    path('eigrp/network/<int:pk>/', include(get_model_urls(app_name, 'eigrpnetwork'))),
    path(
        'eigrp/interface/',
        include(get_model_urls(app_name, 'eigrpinterface', detail=False)),
    ),
    path(
        'eigrp/interface/<int:pk>/', include(get_model_urls(app_name, 'eigrpinterface'))
    ),
    # BFD
    path('bfd/profile/', include(get_model_urls(app_name, 'bfdprofile', detail=False))),
    path('bfd/profile/<int:pk>/', include(get_model_urls(app_name, 'bfdprofile'))),
    # BGP
    path('bgp/setting/', include(get_model_urls(app_name, 'bgpsetting', detail=False))),
    path('bgp/setting/<int:pk>/', include(get_model_urls(app_name, 'bgpsetting'))),
    path(
        'bgp/peer-template/',
        include(get_model_urls(app_name, 'bgppeertemplate', detail=False)),
    ),
    path(
        'bgp/peer-template/<int:pk>/',
        include(get_model_urls(app_name, 'bgppeertemplate')),
    ),
    path(
        'bgp/policy-template/',
        include(get_model_urls(app_name, 'bgppolicytemplate', detail=False)),
    ),
    path(
        'bgp/policy-template/<int:pk>/',
        include(get_model_urls(app_name, 'bgppolicytemplate')),
    ),
    path(
        'bgp/session-template/',
        include(get_model_urls(app_name, 'bgpsessiontemplate', detail=False)),
    ),
    path(
        'bgp/session-template/<int:pk>/',
        include(get_model_urls(app_name, 'bgpsessiontemplate')),
    ),
    path('bgp/router/', include(get_model_urls(app_name, 'bgprouter', detail=False))),
    path('bgp/router/<int:pk>/', include(get_model_urls(app_name, 'bgprouter'))),
    path('bgp/scope/', include(get_model_urls(app_name, 'bgpscope', detail=False))),
    path('bgp/scope/<int:pk>/', include(get_model_urls(app_name, 'bgpscope'))),
    path('bgp/scope/', include(get_model_urls(app_name, 'bgpscope', detail=False))),
    path('bgp/scope/<int:pk>/', include(get_model_urls(app_name, 'bgpscope'))),
    path(
        'bgp/address-family/',
        include(get_model_urls(app_name, 'bgpaddressfamily', detail=False)),
    ),
    path(
        'bgp/address-family/<int:pk>/',
        include(get_model_urls(app_name, 'bgpaddressfamily')),
    ),
    path('bgp/peer/', include(get_model_urls(app_name, 'bgppeer', detail=False))),
    path('bgp/peer/<int:pk>/', include(get_model_urls(app_name, 'bgppeer'))),
    path(
        'bgp/peer-address-family/',
        include(get_model_urls(app_name, 'bgppeeraddressfamily', detail=False)),
    ),
    path(
        'bgp/peer-address-family/<int:pk>/',
        include(get_model_urls(app_name, 'bgppeeraddressfamily')),
    ),
    # Objects: Community
    path(
        'objects/community-list/',
        include(get_model_urls(app_name, 'communitylist', detail=False)),
    ),
    path(
        'objects/community-list/<int:pk>/',
        include(get_model_urls(app_name, 'communitylist')),
    ),
    path(
        'objects/community/',
        include(get_model_urls(app_name, 'community', detail=False)),
    ),
    path('objects/community/<int:pk>/', include(get_model_urls(app_name, 'community'))),
    path(
        'objects/community-list-entry/',
        include(get_model_urls(app_name, 'communitylistentry', detail=False)),
    ),
    path(
        'objects/community-list-entry/<int:pk>/',
        include(get_model_urls(app_name, 'communitylistentry')),
    ),
    # Objects: Custom Prefix
    path(
        'objects/custom-prefix/',
        include(get_model_urls(app_name, 'customprefix', detail=False)),
    ),
    path(
        'objects/custom-prefix/<int:pk>/',
        include(get_model_urls(app_name, 'customprefix')),
    ),
    # Objects: Prefix List
    path(
        'objects/prefix-list/',
        include(get_model_urls(app_name, 'prefixlist', detail=False)),
    ),
    path(
        'objects/prefix-list/<int:pk>/', include(get_model_urls(app_name, 'prefixlist'))
    ),
    path(
        'objects/prefix-list-entry/',
        include(get_model_urls(app_name, 'prefixlistentry', detail=False)),
    ),
    path(
        'objects/prefix-list-entry/<int:pk>/',
        include(get_model_urls(app_name, 'prefixlistentry')),
    ),
    # Objects: Route Map
    path(
        'objects/route-map/',
        include(get_model_urls(app_name, 'routemap', detail=False)),
    ),
    path('objects/route-map/<int:pk>/', include(get_model_urls(app_name, 'routemap'))),
    path(
        'objects/route-map-entry/',
        include(get_model_urls(app_name, 'routemapentry', detail=False)),
    ),
    path(
        'objects/route-map-entry/<int:pk>/',
        include(get_model_urls(app_name, 'routemapentry')),
    ),
    # Objects: AS Path
    path(
        'objects/as-path/',
        include(get_model_urls(app_name, 'aspath', detail=False)),
    ),
    path('objects/as-path/<int:pk>/', include(get_model_urls(app_name, 'aspath'))),
    path(
        'objects/as-path-entry/',
        include(get_model_urls(app_name, 'aspathentry', detail=False)),
    ),
    path(
        'objects/as-path-entry/<int:pk>/',
        include(get_model_urls(app_name, 'aspathentry')),
    ),
]



from typing import Annotated

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType
from netbox_routing import models
from netbox_routing.graphql.eigrp.filters import (
    EIGRPRouterFilter,
    EIGRPAddressFamilyFilter,
    EIGRPNetworkFilter,
    EIGRPInterfaceFilter,
)

__all__ = (
    'EIGRPRouterType',
    'EIGRPAddressFamilyType',
    'EIGRPNetworkType',
    'EIGRPInterfaceType',
)


@strawberry_django.type(models.EIGRPRouter, fields='__all__', filters=EIGRPRouterFilter)
class EIGRPRouterType(PrimaryObjectType):

    device: Annotated["DeviceType", strawberry.lazy('dcim.graphql.types')]  # noqa: F821
    rid: str
    type: str
    name: str
    pid: str


@strawberry_django.type(
    models.EIGRPAddressFamily, fields='__all__', filters=EIGRPAddressFamilyFilter
)
class EIGRPAddressFamilyType(PrimaryObjectType):

    router: Annotated[
        "EIGRPRouterType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    rid: str


@strawberry_django.type(
    models.EIGRPNetwork, fields='__all__', filters=EIGRPNetworkFilter
)
class EIGRPNetworkType(PrimaryObjectType):

    router: Annotated[
        "EIGRPRouterType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    address_family: (
        Annotated[
            "EIGRPAddressFamilyType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    network: Annotated[
        "PrefixType", strawberry.lazy('ipam.graphql.types')
    ]  # noqa: F821


@strawberry_django.type(
    models.EIGRPInterface, fields='__all__', filters=EIGRPInterfaceFilter
)
class EIGRPInterfaceType(PrimaryObjectType):

    router: Annotated[
        "EIGRPRouterType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    address_family: (
        Annotated[
            "EIGRPAddressFamilyType", strawberry.lazy('netbox_routing.graphql.types')
        ]
        | None
    )
    interface: Annotated[
        "InterfaceType", strawberry.lazy('dcim.graphql.types')
    ]  # noqa: F821
    passive: str | None
    bfd: bool | None
    authentication: str | None
    passphrase: str | None



from typing import Annotated

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType
from netbox_routing import models
from netbox_routing.graphql.ospf.filters import (
    OSPFInstanceFilter,
    OSPFAreaFilter,
    OSPFInterfaceFilter,
)

__all__ = (
    'OSPFInstanceType',
    'OSPFAreaType',
    'OSPFInterfaceType',
)


@strawberry_django.type(
    models.OSPFInstance, fields='__all__', filters=OSPFInstanceFilter
)
class OSPFInstanceType(PrimaryObjectType):

    name: str
    device: Annotated["DeviceType", strawberry.lazy('dcim.graphql.types')]
    vrf: Annotated["VRFType", strawberry.lazy('ipam.graphql.types')] | None
    router_id: str
    process_id: str


@strawberry_django.type(models.OSPFArea, fields='__all__', filters=OSPFAreaFilter)
class OSPFAreaType(PrimaryObjectType):

    area_id: str
    area_type: str


@strawberry_django.type(
    models.OSPFInterface, fields='__all__', filters=OSPFInterfaceFilter
)
class OSPFInterfaceType(PrimaryObjectType):

    instance: Annotated[
        "OSPFInstanceType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    area: Annotated["OSPFAreaType", strawberry.lazy('netbox_routing.graphql.types')]
    interface: Annotated["InterfaceType", strawberry.lazy('dcim.graphql.types')]
    passive: bool | None
    priority: str | None
    bfd: bool | None
    authentication: str | None
    passphrase: str | None



from django.utils.translation import gettext_lazy as _

from netbox.plugins import PluginConfig
from importlib.metadata import metadata

plugin = metadata('netbox_routing')


class NetboxRouting(PluginConfig):
    name = plugin.get('Name').replace('-', '_')
    verbose_name = plugin.get('Name').replace('-', ' ').title()
    description = plugin.get('Summary')
    version = plugin.get('Version')
    author = plugin.get('Author')
    author_email = plugin.get('Author-email')
    base_url = 'routing'
    min_version = '4.5.0'
    required_settings = []
    caching_config = {}
    default_settings = {}
    graphql_schema = 'graphql.schema.schema'

    def ready(self):
        from netbox_routing.templatetags.entries import get_entry_url  # noqa: F401

        from netbox_routing.fields.generic import NetBoxGenericRelation
        from dcim.models import Region, SiteGroup, Site, Location, Device
        from ipam.models import Prefix
        from virtualization.models import ClusterGroup, Cluster, VirtualMachine
        from netbox_routing.models import BGPRouter, PrefixListEntry

        super().ready()

        # Add Generic Relations to appropriate models
        models = (
            ('region', Region),
            ('site_group', SiteGroup),
            ('site', Site),
            ('location', Location),
            ('device', Device),
            ('cluster_group', ClusterGroup),
            ('cluster', Cluster),
            ('virtual_machine', VirtualMachine),
        )
        for name, model in models:
            NetBoxGenericRelation(
                to=BGPRouter,
                content_type_field='assigned_object_type',
                object_id_field='assigned_object_id',
                related_name=name,
                related_query_name=name,
            ).contribute_to_class(model, 'bgp_router')

        NetBoxGenericRelation(
            verbose_name=_('Prefix List Entries'),
            to=PrefixListEntry,
            related_name='prefixes',
            related_query_name='prefixes',
            content_type_field='assigned_prefix_type',
            object_id_field='assigned_prefix_id',
        ).contribute_to_class(Prefix, 'prefix_list_entries')


config = NetboxRouting



from netbox_routing.models.objects import *

__all__ = (
    'ParentObjectMixin',
    'ASPathMixin',
    'PrefixListMixin',
    'RouteMapMixin',
)


class ParentObjectMixin:
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()


class ASPathMixin:
    @classmethod
    def setUpTestData(cls):
        cls.aspath = ASPath.objects.create(name=f'{ASPath._meta.verbose_name}')


class PrefixListMixin:
    @classmethod
    def setUpTestData(cls):
        cls.prefix_list = PrefixList.objects.create(
            name=f'{PrefixList._meta.verbose_name}'
        )


class RouteMapMixin:
    @classmethod
    def setUpTestData(cls):
        cls.route_map = RouteMap.objects.create(name=f'{RouteMap._meta.verbose_name}')



from utilities.choices import ChoiceSet


class OSPFAreaTypeChoices(ChoiceSet):
    STANDARD = 'standard'
    BACKBONE = 'backbone'
    STUB = 'stub'
    TSA = 'tsa'
    NSSA = 'nssa'
    TNSSA = 'tnssa'

    CHOICES = [
        (STANDARD, 'Standard Area'),
        (BACKBONE, 'Backbone Area'),
        (STUB, 'Stub Area'),
        (TSA, 'Totally Stubby Area'),
        (NSSA, 'Not-So-Stubby Area'),
        (TNSSA, 'Totally Not-So-Stubby Area'),
    ]



from utilities.choices import ChoiceSet


class ActionChoices(ChoiceSet):
    PERMIT = 'permit'
    DENY = 'deny'

    CHOICES = [(PERMIT, 'Permit', 'blue'), (DENY, 'Deny', 'red')]


class CommunityStatusChoices(ChoiceSet):
    key = "Community.status"

    STATUS_ACTIVE = 'active'
    STATUS_RESERVED = 'reserved'
    STATUS_DEPRECATED = 'deprecated'

    CHOICES = [
        (STATUS_ACTIVE, 'Active', 'blue'),
        (STATUS_RESERVED, 'Reserved', 'cyan'),
        (STATUS_DEPRECATED, 'Deprecated', 'red'),
    ]



from typing import List, Annotated

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType
from netbox_routing import models
from netbox_routing.graphql.static.filters import StaticRouteFilter

__all__ = ('StaticRouteType',)


@strawberry_django.type(models.StaticRoute, fields='__all__', filters=StaticRouteFilter)
class StaticRouteType(PrimaryObjectType):

    name: str
    devices: List[Annotated["DeviceType", strawberry.lazy('dcim.graphql.types')]] | None
    vrf: Annotated["VRFType", strawberry.lazy('ipam.graphql.types')] | None
    prefix: str | None
    next_hop: str | None
    metric: int | None
    permanent: bool | None



from typing import Annotated

import strawberry
import strawberry_django
from strawberry import ID
from strawberry_django import FilterLookup

from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models

__all__ = (
    'ASPathFilter',
    'ASPathEntryFilter',
    'RouteMapFilter',
    'RouteMapEntryFilter',
    'PrefixListFilter',
    'PrefixListEntryFilter',
    'CustomPrefixFilter',
)


@strawberry_django.filter(models.ASPath, lookups=True)
class ASPathFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.ASPathEntry, lookups=True)
class ASPathEntryFilter(PrimaryModelFilter):
    aspath: (
        Annotated[
            'ASPathFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    aspath_id: ID | None = strawberry_django.filter_field()


@strawberry_django.filter(models.PrefixList, lookups=True)
class PrefixListFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.PrefixListEntry, lookups=True)
class PrefixListEntryFilter(PrimaryModelFilter):
    prefix_list: (
        Annotated[
            'PrefixListFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    prefix_list_id: ID | None = strawberry_django.filter_field()


@strawberry_django.filter(models.RouteMap, lookups=True)
class RouteMapFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.RouteMapEntry, lookups=True)
class RouteMapEntryFilter(PrimaryModelFilter):
    route_map: (
        Annotated[
            'RouteMapFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    route_map_id: ID | None = strawberry_django.filter_field()


@strawberry_django.filter(models.CustomPrefix, lookups=True)
class CustomPrefixFilter(PrimaryModelFilter):
    prefix: FilterLookup[str] | None = strawberry_django.filter_field()



from django import forms
from django.utils.translation import gettext as _

from tenancy.models import Tenant
from utilities.forms.fields import DynamicModelChoiceField

__all__ = ('TenantBulkEditMixin',)


class TenantBulkEditMixin(forms.Form):
    tenant = DynamicModelChoiceField(
        label=_('Tenant'), queryset=Tenant.objects.all(), required=False
    )


class EnableMixin(forms.Form):
    enabled = forms.BooleanField(label=_('Enabled'), required=False)



from .bgp.types import *
from .community.types import *
from .eigrp.types import *
from .objects.types import *
from .ospf.types import *
from .static.types import *

__all__ = (
    'ASPathType',
    'ASPathEntryType',
    'BGPPeerTemplateType',
    'BGPPolicyTemplateType',
    'BGPSessionTemplateType',
    'BGPRouterType',
    'BGPScopeType',
    'BGPAddressFamilyType',
    'BGPPeerType',
    'BGPPeerAddressFamilyType',
    'CommunityType',
    'CommunityListType',
    'CommunityListEntryType',
    'EIGRPRouterType',
    'EIGRPAddressFamilyType',
    'EIGRPNetworkType',
    'EIGRPInterfaceType',
    'OSPFInstanceType',
    'OSPFAreaType',
    'OSPFInterfaceType',
    'PrefixListType',
    'PrefixListEntryType',
    'RouteMapType',
    'RouteMapEntryType',
    'StaticRouteType',
)



# Generated by Django 5.0.9 on 2024-11-10 22:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("netbox_routing", "0013_staticroute_tag"),
    ]

    operations = [
        migrations.AddField(
            model_name="ospfarea",
            name="area_type",
            field=models.CharField(default="standard"),
        ),
    ]



from rest_framework import serializers

from dcim.api.serializers_.device_components import InterfaceSerializer
from dcim.api.serializers_.devices import DeviceSerializer
from ipam.api.serializers_.ip import PrefixSerializer
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.models import (
    EIGRPRouter,
    EIGRPAddressFamily,
    EIGRPNetwork,
    EIGRPInterface,
)

__all__ = (
    'EIGRPRouterSerializer',
    'EIGRPAddressFamilySerializer',
    'EIGRPNetworkSerializer',
    'EIGRPInterfaceSerializer',
)


class EIGRPRouterSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:eigrprouter-detail'
    )
    device = DeviceSerializer(nested=True)

    class Meta:
        model = EIGRPRouter
        fields = (
            'url',
            'id',
            'display',
            'name',
            'pid',
            'rid',
            'device',
            'description',
            'comments',
            'custom_fields',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'name',
            'pid',
            'rid',
            'device',
        )


class EIGRPAddressFamilySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:eigrpaddressfamily-detail'
    )
    router = EIGRPRouterSerializer(nested=True)

    class Meta:
        model = EIGRPAddressFamily
        fields = (
            'url',
            'id',
            'display',
            'router',
            'family',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'router',
            'family',
        )


class EIGRPNetworkSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:eigrpnetwork-detail'
    )
    router = EIGRPRouterSerializer(nested=True)
    address_family = EIGRPAddressFamilySerializer(nested=True)
    network = PrefixSerializer(nested=True)

    class Meta:
        model = EIGRPNetwork
        fields = (
            'url',
            'id',
            'display',
            'router',
            'address_family',
            'network',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'router',
            'address_family',
            'network',
        )


class EIGRPInterfaceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:eigrpinterface-detail'
    )
    router = EIGRPRouterSerializer(nested=True)
    address_family = EIGRPAddressFamilySerializer(nested=True)
    interface = InterfaceSerializer(nested=True)

    class Meta:
        model = EIGRPInterface
        fields = (
            'url',
            'id',
            'display',
            'router',
            'address_family',
            'interface',
            'passive',
            'bfd',
            'authentication',
            'passphrase',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'router',
            'address_family',
            'interface',
        )



# Generated by Django 5.2.5 on 2026-02-13 19:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0021_bgp'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgppeer',
            name='status',
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
    ]



from netbox_routing.api._serializers.objects import *
from netbox_routing.api._serializers.community import *
from netbox_routing.api._serializers.static import StaticRouteSerializer
from netbox_routing.api._serializers.bgp import *
from netbox_routing.api._serializers.ospf import *
from netbox_routing.api._serializers.eigrp import *

__all__ = (
    'CommunitySerializer',
    'CommunityListSerializer',
    'CommunityListEntrySerializer',
    'StaticRouteSerializer',
    'OSPFInstanceSerializer',
    'OSPFAreaSerializer',
    'OSPFInterfaceSerializer',
    'EIGRPRouterSerializer',
    'EIGRPAddressFamilySerializer',
    'EIGRPNetworkSerializer',
    'EIGRPInterfaceSerializer',
    'PrefixListSerializer',
    'PrefixListEntrySerializer',
    'RouteMapSerializer',
    'RouteMapEntrySerializer',
    'BGPSettingSerializer',
    'BGPRouterSerializer',
    'BGPScopeSerializer',
    'BGPAddressFamilySerializer',
    'BGPPeerSerializer',
)



# Generated by Django 4.1.10 on 2023-08-09 19:07

from django.db import migrations, models
import django.db.models.deletion
import netbox_routing.fields.ip
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0004_alter_prefixlistentry_ge_alter_prefixlistentry_le'),
    ]

    operations = [
        migrations.CreateModel(
            name='OSPFArea',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('area_id', models.CharField(max_length=100)),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'OSPF Area',
            },
        ),
        migrations.CreateModel(
            name='OSPFInstance',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=100)),
                ('router_id', netbox_routing.fields.ip.IPAddressField()),
                ('process_id', models.IntegerField()),
                (
                    'device',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='ospf_instances',
                        to='dcim.device',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'OSPF Instance',
            },
        ),
        migrations.CreateModel(
            name='OSPFInterface',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('priority', models.IntegerField(blank=True, null=True)),
                ('bfd', models.BooleanField(blank=True, null=True)),
                (
                    'authentication',
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                ('passphrase', models.CharField(blank=True, max_length=200, null=True)),
                (
                    'area',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='interfaces',
                        to='netbox_routing.ospfarea',
                    ),
                ),
                (
                    'instance',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='interfaces',
                        to='netbox_routing.ospfinstance',
                    ),
                ),
                (
                    'interface',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='ospf_interfaces',
                        to='dcim.interface',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'OSPF Interface',
                'ordering': ('instance', 'area', 'interface'),
            },
        ),
        migrations.AddConstraint(
            model_name='ospfinterface',
            constraint=models.UniqueConstraint(
                fields=('interface',),
                name='netbox_routing_ospfinterface_unique_interface',
            ),
        ),
    ]



from django.db.models import ForeignKey

from ipam.models import Prefix
from utilities.testing import ViewTestCases

from netbox_routing.models.objects import *
from netbox_routing.tests.base import (
    AutomatedModelCreationMixin,
    AutomatedFormDataCreationMixin,
    BulkEditMixin,
)
from netbox_routing.tests.objects.base import *

__all__ = (
    'ASPathTestCase',
    'ASPathEntryTestCase',
    'PrefixListTestCase',
    'PrefixListEntryTestCase',
    'RouteMapTestCase',
    'RouteMapEntryTestCase',
)


class NBRObjectMixin:

    @classmethod
    def setUpTestData(cls):
        name = cls.model._meta.verbose_name
        items = []

        for seq in range(1, 4):
            items.append(
                cls.model(
                    name=f'{name} {seq}',
                ),
            )
        cls.model.objects.bulk_create(items)

        cls.form_data = {
            'name': f'{name} 4',
        }

        cls.bulk_edit_data = {
            'description': 'Test Description',
        }


class NBRObjectEntryMixin:

    @classmethod
    def setUpTestData(cls):
        parent_field = None
        parent = None
        for parent_field in cls.model._meta.fields:
            if isinstance(parent_field, ForeignKey) and parent_field.name != 'owner':
                parent = parent_field.related_model(
                    name=parent_field.related_model._meta.verbose_name
                )
                parent.full_clean()
                parent.save()
                break

        items = []
        for seq in range(1, 4):
            args = {
                parent_field.name: parent,
                'action': 'permit',
                'sequence': seq,
                cls.key_field: cls.key_value,
            }
            items.append(
                cls.model(**args),
            )
        cls.model.objects.bulk_create(items)

        cls.form_data = {
            parent_field.name: parent.pk,
            'action': 'permit',
            'sequence': 9999,
            cls.key_field: cls.key_value,
        }

        cls.bulk_edit_data = {
            'description': 'Test Description',
        }


class ASPathTestCase(
    AutomatedModelCreationMixin,
    AutomatedFormDataCreationMixin,
    BulkEditMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = ASPath

    routing_required_fields = ('name',)

    def _get_base_url(self):
        return 'plugins:netbox_routing:aspath_{}'


class ASPathEntryTestCase(
    AutomatedModelCreationMixin,
    AutomatedFormDataCreationMixin,
    BulkEditMixin,
    ASPathMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = ASPathEntry

    action = 'permit'
    pattern = '^.*$'
    routing_required_fields = ('aspath', 'pattern', 'sequence', 'action')

    def _get_base_url(self):
        return 'plugins:netbox_routing:aspathentry_{}'


class PrefixListTestCase(
    AutomatedModelCreationMixin,
    AutomatedFormDataCreationMixin,
    BulkEditMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = PrefixList

    family = 4
    routing_required_fields = (
        'name',
        'family',
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def _get_base_url(self):
        return 'plugins:netbox_routing:prefixlist_{}'


class PrefixListEntryTestCase(
    AutomatedModelCreationMixin,
    AutomatedFormDataCreationMixin,
    BulkEditMixin,
    PrefixListMixin,
    NBRObjectEntryMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = PrefixListEntry

    action = 'permit'
    routing_required_fields = (
        'prefix_list',
        ('assigned_prefix', 'assigned_prefixes'),
        'sequence',
        'action',
    )

    def _get_base_url(self):
        return 'plugins:netbox_routing:prefixlistentry_{}'

    @classmethod
    def setUpTestData(cls):
        cls.prefix = Prefix.objects.create(prefix='10.0.0.0/8')
        cls.custom_prefix = CustomPrefix.objects.create(prefix='10.0.0.0/8')

        cls.assigned_prefixes = []
        for i in range(1, 11):
            if i % 2 == 0:
                cls.assigned_prefixes.append(cls.prefix)
            else:
                cls.assigned_prefixes.append(cls.custom_prefix)

        super().setUpTestData()

        cls.form_data['custom_prefix'] = None
        cls.form_data['ipam_prefix'] = cls.form_data['assigned_prefix'].pk
        del cls.form_data['assigned_prefix']


class RouteMapTestCase(
    AutomatedFormDataCreationMixin,
    AutomatedModelCreationMixin,
    BulkEditMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = RouteMap
    routing_required_fields = ('name',)

    def _get_base_url(self):
        return 'plugins:netbox_routing:routemap_{}'


class RouteMapEntryTestCase(
    AutomatedFormDataCreationMixin,
    AutomatedModelCreationMixin,
    RouteMapMixin,
    BulkEditMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = RouteMapEntry
    action = 'permit'
    set = {}
    match = {
        'tags': 1234,
    }
    routing_required_fields = ('match', 'set', 'route_map', 'action', 'sequence')

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

    def _get_base_url(self):
        return 'plugins:netbox_routing:routemapentry_{}'



from netbox.search import SearchIndex, register_search
from netbox_routing.models.bgp import *


@register_search
class BGPPeerTemplateIndex(SearchIndex):
    model = BGPPeerTemplate
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class BGPPolicyTemplateIndex(SearchIndex):
    model = BGPPolicyTemplate
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class BGPSessionTemplateIndex(SearchIndex):
    model = BGPSessionTemplate
    fields = (
        ('name', 100),
        ('parent', 200),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class BGPSettingIndex(SearchIndex):
    model = BGPSetting
    fields = (
        ('search_display_name', 50),
        ('key', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'key',
        'assigned_object',
    )


@register_search
class BGPRouterIndex(SearchIndex):
    model = BGPRouter
    fields = (
        ('search_display_name', 50),
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'assigned_object',
        'name',
    )


@register_search
class BGPScopeIndex(SearchIndex):
    model = BGPScope
    fields = (
        ('router', 100),
        ('vrf', 200),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'router',
        'vrf',
    )


@register_search
class BGPAddressFamilyIndex(SearchIndex):
    model = BGPAddressFamily
    fields = (
        ('scope', 100),
        ('address_family', 500),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'scope',
        'address_family',
    )


@register_search
class BGPPeerIndex(SearchIndex):
    model = BGPPeer
    fields = (
        ('name', 100),
        ('peer', 200),
        ('remote_as', 210),
        ('source', 300),
        ('local_as', 310),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'name',
        'peer',
        'remote_as',
    )


@register_search
class BGPPeerAddressFamilyIndex(SearchIndex):
    model = BGPPeerAddressFamily
    fields = (
        ('search_display_name', 50),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'assigned_object',
        'address_family',
    )



from typing import Annotated

import strawberry
import strawberry_django
from strawberry import ID


class DeviceMixin:
    device: (
        Annotated['DeviceFilter', strawberry.lazy('dcim.graphql.filters')] | None
    ) = strawberry_django.filter_field()
    device_id: ID | None = strawberry_django.filter_field()


class InterfaceMixin(DeviceMixin):
    device: (
        Annotated['DeviceFilter', strawberry.lazy('dcim.graphql.filters')] | None
    ) = strawberry_django.filter_field()
    device_id: ID | None = strawberry_django.filter_field()
    interface: (
        Annotated['InterfaceFilter', strawberry.lazy('dcim.graphql.filters')] | None
    ) = strawberry_django.filter_field()
    interface_id: ID | None = strawberry_django.filter_field()


class VRFMixin:
    vrf: Annotated['VRFFilter', strawberry.lazy('ipam.graphql.filters')] | None = (
        strawberry_django.filter_field()
    )


class NetworkPrefixMixin:
    network: (
        Annotated['PrefixFilter', strawberry.lazy('ipam.graphql.filters')] | None
    ) = strawberry_django.filter_field()
    network_id: ID | None = strawberry_django.filter_field()



from netbox.search import SearchIndex, register_search
from netbox_routing.models.community import *


@register_search
class CommunityIndex(SearchIndex):
    model = Community
    fields = (
        ('community', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('community',)


@register_search
class CommunityListIndex(SearchIndex):
    model = CommunityList
    fields = (
        ('name', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = ('name',)


@register_search
class CommunityListEntryIndex(SearchIndex):
    model = CommunityListEntry
    fields = (
        ('community_list', 100),
        ('community', 100),
        ('description', 4000),
        ('comments', 5000),
    )
    display_attrs = (
        'community_list',
        'community',
    )



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable, columns
from netbox_routing.models.objects import *

__all__ = (
    'ASPathTable',
    'ASPathEntryTable',
    'CustomPrefixTable',
    'PrefixListTable',
    'PrefixListEntryTable',
    'RouteMapTable',
    'RouteMapEntryTable',
)


class PrefixListTable(NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))
    family = tables.Column(linkify=True, verbose_name=_('Family'))

    class Meta(NetBoxTable.Meta):
        model = PrefixList
        fields = ('pk', 'id', 'name', 'family')
        default_columns = ('pk', 'id', 'name')


class PrefixListEntryTable(NetBoxTable):
    prefix_list = tables.Column(verbose_name=_('Prefix List'), linkify=True)
    action = columns.ChoiceFieldColumn()

    class Meta(NetBoxTable.Meta):
        model = PrefixListEntry
        fields = (
            'pk',
            'id',
            'prefix_list',
            'sequence',
            'action',
            'assigned_prefix',
            'le',
            'ge',
        )
        default_columns = (
            'pk',
            'id',
            'prefix_list',
            'sequence',
            'action',
            'assigned_prefix',
            'le',
            'ge',
        )


class CustomPrefixTable(NetBoxTable):

    class Meta(NetBoxTable.Meta):
        model = CustomPrefix
        fields = (
            'pk',
            'id',
            'prefix',
        )
        default_columns = (
            'pk',
            'id',
            'prefix',
        )


class RouteMapTable(NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))

    class Meta(NetBoxTable.Meta):
        model = RouteMap
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class RouteMapEntryTable(NetBoxTable):
    route_map = tables.Column(verbose_name=_('Route Map'), linkify=True)
    action = columns.ChoiceFieldColumn()
    flow_control = tables.Column(verbose_name=_('Flow Control (Continue)'))

    class Meta(NetBoxTable.Meta):
        model = RouteMapEntry
        fields = ('pk', 'id', 'route_map', 'sequence', 'action', 'flow_control')
        default_columns = ('pk', 'id', 'route_map', 'sequence', 'action')


class ASPathTable(NetBoxTable):
    name = tables.Column(linkify=True, verbose_name=_('Name'))

    class Meta(NetBoxTable.Meta):
        model = ASPath
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class ASPathEntryTable(NetBoxTable):
    aspath = tables.Column(verbose_name=_('AS Path'), linkify=True)
    action = columns.ChoiceFieldColumn()
    asn = tables.Column(verbose_name=_('ASN'), linkify=True)

    class Meta(NetBoxTable.Meta):
        model = ASPathEntry
        fields = (
            'pk',
            'id',
            'aspath',
            'sequence',
            'action',
            'asn',
        )
        default_columns = (
            'pk',
            'id',
            'aspath',
            'sequence',
            'action',
            'asn',
        )



from ipam.models import Role
from netbox_routing.choices import ActionChoices
from utilities.testing import ViewTestCases
from tenancy.models import Tenant

from netbox_routing.models.community import *

__all__ = (
    'CommunityTestCase',
    'CommunityListTestCase',
    'CommunityListEntryTestCase',
)


class CommunityTestCase(
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = Community

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')
        cls.role = Role.objects.create(name='Role 1')

        cls.communities = (
            Community(
                community='64512',
                status='active',
            ),
            Community(
                community='64513',
                status='active',
                role=cls.role,
            ),
            Community(
                community='64514',
                status='active',
                role=cls.role,
                tenant=cls.tenant,
            ),
        )

        Community.objects.bulk_create(cls.communities)

        cls.form_data = {
            'community': '64515',
            'status': 'active',
        }

        cls.bulk_edit_data = {
            'description': 'Test Description',
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:community_{}'


class CommunityListTestCase(
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = CommunityList

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')

        cls.community_list = (
            CommunityList(
                name='Community List 1',
            ),
            CommunityList(
                name='Community List 2',
            ),
            CommunityList(
                name='Community List 3',
                tenant=cls.tenant,
            ),
        )

        CommunityList.objects.bulk_create(cls.community_list)

        cls.form_data = {
            'name': 'Community List X',
        }

        cls.bulk_edit_data = {
            'description': 'Test Description',
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:communitylist_{}'


class CommunityListEntryTestCase(
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = CommunityListEntry

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')
        cls.role = Role.objects.create(name='Role 1')

        cls.communities = (
            Community(
                community='64512',
                status='active',
            ),
            Community(
                community='64513',
                status='active',
                role=cls.role,
            ),
            Community(
                community='64514',
                status='active',
                role=cls.role,
                tenant=cls.tenant,
            ),
        )

        Community.objects.bulk_create(cls.communities)

        cls.community_lists = (
            CommunityList(
                name='Community List 1',
            ),
            CommunityList(
                name='Community List 2',
            ),
            CommunityList(
                name='Community List 3',
                tenant=cls.tenant,
            ),
        )

        CommunityList.objects.bulk_create(cls.community_lists)

        cls.community_list_entries = (
            CommunityListEntry(
                community_list=cls.community_lists[0],
                community=cls.communities[0],
                action=ActionChoices.PERMIT,
            ),
            CommunityListEntry(
                community_list=cls.community_lists[0],
                community=cls.communities[0],
                action=ActionChoices.PERMIT,
            ),
            CommunityListEntry(
                community_list=cls.community_lists[1],
                community=cls.communities[0],
                action=ActionChoices.PERMIT,
            ),
            CommunityListEntry(
                community_list=cls.community_lists[1],
                community=cls.communities[1],
                action=ActionChoices.PERMIT,
            ),
        )

        CommunityListEntry.objects.bulk_create(cls.community_list_entries)

        cls.form_data = {
            'community_list': cls.community_lists[2].pk,
            'community': cls.communities[2].pk,
            'action': ActionChoices.PERMIT,
        }

        cls.bulk_edit_data = {
            'description': 'Test Description',
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:communitylistentry_{}'



from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import gettext as _

from netbox.models import PrimaryModel

from netbox_routing.choices import ActionChoices, CommunityStatusChoices

__all__ = (
    'CommunityList',
    'Community',
    'CommunityListEntry',
)


class CommunityList(PrimaryModel):
    name = models.CharField(
        verbose_name=_('List'),
        max_length=100,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='community_lists',
        null=True,
        blank=True,
    )

    class Meta:
        verbose_name = _('Community List')
        verbose_name_plural = 'Community Lists'
        unique_together = ['name']
        ordering = ['name']

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        from django.urls import reverse

        return reverse('plugins:netbox_routing:communitylist', args=[self.pk])


class Community(PrimaryModel):
    community = models.CharField(
        verbose_name=_('Community'),
        max_length=255,
        validators=[RegexValidator(r'^[\d\.]+(?::[\d\.]+(?::[\d\.]+)?)?$')],
    )

    status = models.CharField(
        max_length=50,
        choices=CommunityStatusChoices,
        default=CommunityStatusChoices.STATUS_ACTIVE,
    )
    role = models.ForeignKey(
        verbose_name=_('Role'),
        to='ipam.Role',
        on_delete=models.SET_NULL,
        related_name='communities',
        null=True,
        blank=True,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='communities',
        null=True,
        blank=True,
    )

    class Meta:
        verbose_name = _('Community')
        verbose_name_plural = 'Communities'
        ordering = ['community']
        constraints = [
            models.UniqueConstraint(
                fields=('community',),
                name='%(app_label)s_%(class)s_community',
            ),
        ]

    def __str__(self):
        return f'{self.community}'

    def get_absolute_url(self):
        from django.urls import reverse

        return reverse('plugins:netbox_routing:community', args=[self.pk])

    def get_status_color(self):
        return CommunityStatusChoices.colors.get(self.status)


class CommunityListEntry(PrimaryModel):
    """ """

    community_list = models.ForeignKey(
        to=CommunityList, on_delete=models.CASCADE, related_name='communitylistentries'
    )
    action = models.CharField(max_length=30, choices=ActionChoices)
    community = models.ForeignKey(
        to=Community,
        related_name='communitylistentries',
        on_delete=models.CASCADE,
    )

    class Meta:
        ordering = ('community_list', 'community')

    def __str__(self):
        return f'{self.community_list}: {self.action} {self.community}'

    def get_action_color(self):
        return ActionChoices.colors.get(self.action)

    def get_absolute_url(self):
        from django.urls import reverse

        return reverse('plugins:netbox_routing:communitylistentry', args=[self.pk])
